require 'net/http'
require 'openssl'
require 'json'

class CycuCsysBookmarker
  def initialize(student_id, password)
    @student_id = student_id
    @password = password
    @http = Net::HTTP.new('csys.cycu.edu.tw')
    @loggedin = false
    init_cookie()
  end

  def login
    return false if @loggedin
    init_secure_random()
    resp = @http.post('/sso/sso.srv', "cmd=login&userid=#{@student_id}&hash=#{login_hash}", {'Cookie' => @cookie})
    resp_json = JSON.parse(resp.body)

    raise resp_json['message'] unless resp_json['result']

    @page_id = resp_json['pageId']
    @loggedin = true
  end

  def logout
    return false unless @loggedin
    resp = @http.post('/sso/sso.srv', 'cmd=logout', {'Cookie' => @cookie})
    @loggedin = false
  end

  def bookmark(list = [])
    return false unless @loggedin
    headers = { 'Cookie' => @cookie, 'Page-Id' => @page_id }
    path = '/student/op/StudentCourseTrace.srv'

    list.each do |item|
      resp = @http.post(path, "cmd=insert&op_code=#{item}", headers)
    end
  end

  private

  def init_cookie
    resp = @http.get('/index.jsp')
    raw_cookies = resp.response['set-cookie']

    cookies = []
    raw_cookies.split(/, ?/).each do |raw_cookie|
      cookies << raw_cookie.split(';')[0]
    end
    @cookie = cookies.join(';')
  end

  def init_secure_random
    resp = @http.post('/sso/sso.srv', 'cmd=login_init', {'Cookie' => @cookie})
    resp_json = JSON.parse(resp.body)
    @secure_random = resp_json['secureRandom']
  end

  def login_hash
    OpenSSL::HMAC.hexdigest('sha256', @password, @student_id + @secure_random)
  end
end